#include "bdialog.h"

BasicDialog::BasicDialog(string dtitle, int _bnumber, int closes, bool modal, bool m_or_c){
  bnumber = _bnumber;
  closestate = closes;

  basicdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (basicdialog), dtitle.c_str());
  gtk_window_set_resizable (GTK_WINDOW (basicdialog), FALSE);
  
  if(m_or_c){
    gtk_window_set_position (GTK_WINDOW (basicdialog), GTK_WIN_POS_CENTER);
  }
  else{
    gtk_window_set_position (GTK_WINDOW (basicdialog), GTK_WIN_POS_MOUSE);
  }
  
  if(modal){
    gtk_window_set_modal (GTK_WINDOW (basicdialog), TRUE);
  }

  dialog_vbox = GTK_DIALOG (basicdialog)->vbox;
  gtk_widget_show (dialog_vbox);
  
  dialog_action_area = GTK_DIALOG (basicdialog)->action_area;
  gtk_widget_show (dialog_action_area);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_SPREAD);  
}

BasicDialog::~BasicDialog(){
  gtk_widget_destroy(basicdialog);  
}

void BasicDialog::requestSize(int x, int y){
  gtk_widget_set_size_request (basicdialog, x, y);
}

void BasicDialog::setTransientFor(GtkWindow *theWindow){
  gtk_window_set_transient_for(GTK_WINDOW(basicdialog), GTK_WINDOW(theWindow));
}

void BasicDialog::showDialog(){
  gtk_widget_show(basicdialog);
}

int BasicDialog::createDialog(string dimage, string dlabel){
  if(bnumber == b_ok){
    return createDialog(dimage,dlabel, GTK_STOCK_OK, "_OK", "", "");
  }
  if(bnumber == b_both){
    return createDialog(dimage, dlabel, GTK_STOCK_CANCEL, "_Cancel" , GTK_STOCK_OK, "_OK");
  }
  if(bnumber == b_cancel){
    return createDialog(dimage, dlabel, GTK_STOCK_CANCEL, "_Cancel" , "", "");
  }
  return -1;
}

int BasicDialog::createDialog(string dimage, string dlabel, string bimage, string blabel, 
			      int type, int adjust){ 
  if(bnumber == b_both){
    if(adjust == b_right){
      if(type == b_ok){
	return createDialog(dimage,dlabel, GTK_STOCK_OK, "_OK", bimage, blabel);
      }
      if(type == b_cancel){
	return createDialog(dimage, dlabel, GTK_STOCK_CANCEL, "_Cancel" , bimage, blabel);
      }
    }
    else if(adjust == b_left){
      if(type == b_ok){
	return createDialog(dimage,dlabel, bimage, blabel, GTK_STOCK_OK, "_OK");
      }
      if(type == b_cancel){
	return createDialog(dimage, dlabel, bimage, blabel, GTK_STOCK_CANCEL, "_Cancel");
      }
    }
  }
  else{
    return createDialog(dimage, dlabel, bimage, blabel, "", "");
  }
}

int BasicDialog::createDialog(string dimage, string dlabel, string b1image, 
			      string b1label, string b2image, string b2label){
  
  if(dimage != "none"){
    dialog_hbox = gtk_hbox_new (FALSE, 0);
  }
  else{
    dialog_hbox = gtk_hbox_new (TRUE, 0);
  }
  
  gtk_widget_show (dialog_hbox);
  gtk_box_pack_start (GTK_BOX (dialog_vbox), dialog_hbox, TRUE, TRUE, 0);

  if(dimage != "none"){
    dialog_image = gtk_image_new_from_stock (dimage.c_str(), GTK_ICON_SIZE_DIALOG);
    gtk_widget_show (dialog_image);
    gtk_box_pack_start (GTK_BOX (dialog_hbox), dialog_image, TRUE, TRUE, 0);
  }

  dialog_label = gtk_label_new (dlabel.c_str());
  gtk_widget_show (dialog_label);
  gtk_box_pack_start (GTK_BOX (dialog_hbox), dialog_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (dialog_label), GTK_JUSTIFY_CENTER);

  dialog_button1 = gtk_button_new ();
  gtk_widget_show (dialog_button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (basicdialog), dialog_button1, GTK_RESPONSE_CANCEL);
  gtk_widget_grab_focus(dialog_button1);

  button_alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (button_alignment1);
  gtk_container_add (GTK_CONTAINER (dialog_button1), button_alignment1);

  button_hbox1 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (button_hbox1);
  gtk_container_add (GTK_CONTAINER (button_alignment1), button_hbox1);

  button_image1 = gtk_image_new_from_stock (b1image.c_str(), GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (button_image1);
  gtk_box_pack_start (GTK_BOX (button_hbox1), button_image1, FALSE, FALSE, 0);

  button_label1 = gtk_label_new_with_mnemonic (b1label.c_str());
  gtk_widget_show (button_label1);
  gtk_box_pack_start (GTK_BOX (button_hbox1), button_label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (button_label1), GTK_JUSTIFY_LEFT);
  
  if(bnumber == b_both){
    dialog_button2 = gtk_button_new ();
    gtk_widget_show (dialog_button2);
    gtk_dialog_add_action_widget (GTK_DIALOG (basicdialog), dialog_button2, GTK_RESPONSE_CANCEL);
    gtk_widget_grab_focus(dialog_button2);
    
    button_alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
    gtk_widget_show (button_alignment2);
    gtk_container_add (GTK_CONTAINER (dialog_button2), button_alignment2);
    
    button_hbox2 = gtk_hbox_new (FALSE, 2);
    gtk_widget_show (button_hbox2);
    gtk_container_add (GTK_CONTAINER (button_alignment2), button_hbox2);
    
    button_image2 = gtk_image_new_from_stock (b2image.c_str(), GTK_ICON_SIZE_BUTTON);
    gtk_widget_show (button_image2);
    gtk_box_pack_start (GTK_BOX (button_hbox2), button_image2, FALSE, FALSE, 0);

    button_label2 = gtk_label_new_with_mnemonic (b2label.c_str());
    gtk_widget_show (button_label2);
    gtk_box_pack_start (GTK_BOX (button_hbox2), button_label2, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (button_label2), GTK_JUSTIFY_LEFT);    
  }

  switch(closestate){
  case c_close:
    g_signal_connect((gpointer) basicdialog, "delete-event",
		     G_CALLBACK (dialog_delete),
		     this);
    break;
  case c_closenot:
    g_signal_connect((gpointer) basicdialog, "delete-event",
		     G_CALLBACK (gtk_true),
		     this);
    break;
  default:
    break;
  }
}

int BasicDialog::connectSignal(gpointer sobject, string sname, GCallback fptr){
  g_signal_connect(sobject, sname.c_str(), fptr, this);
}

void dialog_delete(GtkWidget *dialog, GdkEvent *event, gpointer user_data){
  delete (BasicDialog*)user_data;
}

void button_delete(GtkButton *button, gpointer user_data){
  delete (BasicDialog*)user_data;
}
