#include "editdiscdialog.h"
#include "discutils.h"
#include "project.h"

extern Project *project;

EditDisc::EditDisc(string dname, int dtype, int dsize){
  discName = dname;
  discType = dtype;
  discSize = dsize;
}

EditDisc::~EditDisc(){
  gtk_widget_destroy(editdiscdialog);
}

void EditDisc::createDialog(){
  editdiscdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (editdiscdialog), "Disc Properties");
  gtk_window_set_position (GTK_WINDOW (editdiscdialog), GTK_WIN_POS_MOUSE);

  dialog_vbox = GTK_DIALOG (editdiscdialog)->vbox;
  gtk_widget_show (dialog_vbox);

  top_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (top_hbox);
  gtk_box_pack_start (GTK_BOX (dialog_vbox), top_hbox, TRUE, TRUE, 0);

  cdromimage = gtk_image_new_from_stock ("gtk-cdrom", GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (cdromimage);
  gtk_box_pack_start (GTK_BOX (top_hbox), cdromimage, TRUE, TRUE, 0);

  top_table = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (top_table);
  gtk_box_pack_start (GTK_BOX (top_hbox), top_table, TRUE, TRUE, 0);

  disc_label = gtk_label_new ("Label:");
  gtk_widget_show (disc_label);
  gtk_table_attach (GTK_TABLE (top_table), disc_label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (disc_label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (disc_label), 0, 0.5);

  editdisc_entry = gtk_entry_new ();
  gtk_widget_show (editdisc_entry);
  gtk_table_attach (GTK_TABLE (top_table), editdisc_entry, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  size_label = gtk_label_new ("Size:");
  gtk_widget_show (size_label);
  gtk_table_attach (GTK_TABLE (top_table), size_label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (size_label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (size_label), 0, 0.5);

  sizemenu = gtk_option_menu_new ();
  gtk_widget_show (sizemenu);
  gtk_table_attach (GTK_TABLE (top_table), sizemenu, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  menu_size = gtk_menu_new ();

  _700MB = gtk_menu_item_new_with_mnemonic ("700 mb");
  gtk_widget_show (_700MB);
  gtk_container_add (GTK_CONTAINER (menu_size), _700MB);

  _650MB = gtk_menu_item_new_with_mnemonic ("650 mb");
  gtk_widget_show (_650MB);
  gtk_container_add (GTK_CONTAINER (menu_size), _650MB);

  _4700MB = gtk_menu_item_new_with_mnemonic ("4700 mb");
  gtk_widget_show (_4700MB);
  gtk_container_add (GTK_CONTAINER (menu_size), _4700MB);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (sizemenu), menu_size);

  type_label = gtk_label_new ("Type:");
  gtk_widget_show (type_label);
  gtk_table_attach (GTK_TABLE (top_table), type_label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (type_label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (type_label), 0, 0.5);

  typemenu = gtk_option_menu_new ();
  gtk_widget_show (typemenu);
  gtk_table_attach (GTK_TABLE (top_table), typemenu, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  menu_type = gtk_menu_new ();

  type_cdrw_item = gtk_menu_item_new_with_mnemonic ("CD-RW");
  gtk_widget_show (type_cdrw_item);
  gtk_container_add (GTK_CONTAINER (menu_type), type_cdrw_item);

  type_cdr_item = gtk_menu_item_new_with_mnemonic ("CD-R");
  gtk_widget_show (type_cdr_item);
  gtk_container_add (GTK_CONTAINER (menu_type), type_cdr_item);

  type_dvdr_item = gtk_menu_item_new_with_mnemonic ("DVD-R");
  gtk_widget_show (type_dvdr_item);
  gtk_container_add (GTK_CONTAINER (menu_type), type_dvdr_item);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (typemenu), menu_type);

  dialog_action_area = GTK_DIALOG (editdiscdialog)->action_area;
  gtk_widget_show (dialog_action_area);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_SPREAD);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (editdiscdialog), cancelbutton, GTK_RESPONSE_CANCEL);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (editdiscdialog), okbutton, GTK_RESPONSE_OK);

  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
  gtk_widget_grab_focus (okbutton);
  gtk_widget_grab_default (okbutton);

  gtk_entry_set_text(GTK_ENTRY(editdisc_entry), discName.c_str());
  
  switch(discSize){
  case 700:
    gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 0);
    break;
  case 650:
    gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 1);
    break;
  case 4700:
    gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 2);
    break;
  }

  if(discSize == 4700){
    gtk_option_menu_set_history(GTK_OPTION_MENU(typemenu), t_dvdr);
  }
  else{
    gtk_option_menu_set_history(GTK_OPTION_MENU(typemenu), discType);
  }
  
  g_signal_connect ((gpointer) _700MB, "activate",
                    G_CALLBACK (on_700mb_activate),this);
  g_signal_connect ((gpointer) _650MB, "activate",
                    G_CALLBACK (on_650mb_activate),this);
  g_signal_connect ((gpointer) _4700MB, "activate",
                    G_CALLBACK (on_4700mb_activate),this);
  g_signal_connect ((gpointer) type_cdrw_item, "activate",
                    G_CALLBACK (on_type_cdrw_item_activate),this);
  g_signal_connect ((gpointer) type_cdr_item, "activate",
                    G_CALLBACK (on_type_cdr_item_activate),this);
  g_signal_connect ((gpointer) type_dvdr_item, "activate",
                    G_CALLBACK (on_type_dvdr_item_activate),this);
  g_signal_connect ((gpointer) cancelbutton, "clicked",
                    G_CALLBACK (on_edit_cancel_clicked),this);
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (on_edit_ok_clicked),this);
  g_signal_connect ((gpointer) editdiscdialog, "delete_event",
		    G_CALLBACK (on_edit_delete_clicked),this);
}

void EditDisc::showDialog(){
  gtk_widget_show(editdiscdialog);
}

void on_650mb_activate(GtkMenuItem *menuitem, gpointer user_data){
  if(project){
    project->discView->newdiscsize = 650;
    if(project->discView->newdisctype == t_dvdr){
      project->discView->newdisctype = t_cdrw;
      GtkWidget *typemenu = ((EditDisc*)user_data)->typemenu;
      gtk_option_menu_set_history(GTK_OPTION_MENU(typemenu), 0);
    }
  }
}

void on_700mb_activate(GtkMenuItem *menuitem, gpointer user_data){
  if(project){
    project->discView->newdiscsize = 700;
    if(project->discView->newdisctype == t_dvdr){
      project->discView->newdisctype = t_cdrw;
      GtkWidget *typemenu = ((EditDisc*)user_data)->typemenu;
      gtk_option_menu_set_history(GTK_OPTION_MENU(typemenu), 0);
    } 
  }
}

void on_4700mb_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  if(project){
    project->discView->newdiscsize = 4700;
    project->discView->newdisctype = t_dvdr;
    GtkWidget *typemenu = ((EditDisc*)user_data)->typemenu;
    gtk_option_menu_set_history(GTK_OPTION_MENU(typemenu), 2);
  }
}

void on_type_cdr_item_activate(GtkMenuItem *menuitem, gpointer user_data){
  if(project){
    project->discView->newdisctype = t_cdr;
    if(project->discView->newdiscsize == 4700){
      project->discView->newdiscsize = 700;
      GtkWidget *sizemenu = ((EditDisc*)user_data)->sizemenu;
      gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 0);
    }
  }
}


void on_type_cdrw_item_activate(GtkMenuItem *menuitem, gpointer user_data){
  if(project){
    project->discView->newdisctype = t_cdrw;
    if(project->discView->newdiscsize == 4700){
      project->discView->newdiscsize = 700;
      GtkWidget *sizemenu = ((EditDisc*)user_data)->sizemenu;
      gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 0);
    }
  }
}

void on_type_dvdr_item_activate(GtkMenuItem *menuitem, gpointer user_data){
  if(project){
    project->discView->newdisctype = t_dvdr;
    project->discView->newdiscsize = 4700;
    GtkWidget *sizemenu = ((EditDisc*)user_data)->sizemenu;  
    gtk_option_menu_set_history(GTK_OPTION_MENU(sizemenu), 2);
  }
}
 
void on_edit_cancel_clicked(GtkButton *button, gpointer user_data){
  delete (EditDisc*)user_data;
  if(project){
    if(project->discView->curdiscnew){
      project->discView->removeDisc();
    }
    project->discView->newdisctype = t_cdrw;
    project->discView->newdiscsize = 700;
    project->discView->curdiscnew = 0;
  }
}


void on_edit_ok_clicked(GtkButton *button, gpointer user_data){
  GtkWidget *edentry = ((EditDisc*)user_data)->editdisc_entry;
  string tmpStr = gtk_entry_get_text(GTK_ENTRY(edentry));
  delete (EditDisc*)user_data;

  if(project){
    Disc *tmpDisc = project->discView->getActiveDisc();
    tmpDisc->setName(tmpStr);
    tmpDisc->setSize(project->discView->newdiscsize, false);
    tmpDisc->setType(project->discView->newdisctype);
    project->discView->newdiscsize = 700;
    project->discView->newdisctype = t_cdrw;
    project->discView->curdiscnew = 0;
  }
}

void on_edit_delete_clicked(GtkWidget *dialog, GdkEvent *event, gpointer user_data){
  delete (EditDisc*)user_data;
  if(project){
    if(project->discView->curdiscnew){
      project->discView->removeDisc();
    }
    project->discView->newdisctype = t_cdrw;
    project->discView->newdiscsize = 700;
    project->discView->curdiscnew = 0;
  }
}

EditDisc *editdiscdialog(string dname, int dtype, int dsize){
  EditDisc *eddialog = new EditDisc(dname, dtype, dsize);
  eddialog->createDialog();
  eddialog->showDialog();
  return eddialog;
}
