#include <gtk/gtk.h>

#include "outofspacedialog.h"
#include "folderutils.h"
#include "discutils.h"
#include "project.h"
#include "math.h"
#include <sstream>

using namespace std;

extern GtkWidget *mainWindow;
extern Project *project;

enum {
  CHECK_COLUMN1=0,
  DISCIMAGE_COLUMN2=1,
  DISCASSIGN_COLUMN3=2,
  FOLDERIMAGE_COLUMN4=3,
  FOLDER_COLUMN5=4
};

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

void toggle_event(GtkCellRendererToggle *cellrenderertoggle, gchar *arg1, gpointer user_data){
  if(project){
    project->folderView->folderToggleEvent(arg1);
  }
}

void FolderView::folderToggleEvent(char *user_data){
 
  Disc *curDisc = project->discView->getActiveDisc();
  if(curDisc == NULL){
    return;
  }
  
  GtkTreePath *thePath;
  thePath = gtk_tree_path_new_from_string(user_data);
  
  GtkTreeIter iter;
  gtk_tree_model_get_iter(model, &iter, thePath);
    
  GValue gValue = {false};
  
  gtk_tree_model_get_value(model, &iter, CHECK_COLUMN1, &gValue);
  
  string svalue = g_strdup_value_contents(&gValue);
  
  bool bvalue;
  
  if(svalue == "TRUE"){
    bvalue = false;
  }
  else{
    bvalue = true;
  }
  
  Folder *tmpFolder = findIter(iter);
  if(tmpFolder){
    if(!tmpFolder->toggleSelectionState(bvalue, false)){
      if(bvalue){
	if(curDisc->haveSpaceForFolder(tmpFolder)){
	  curDisc->addFolder(tmpFolder);
	}
	else{
	  tmpFolder->toggleSelectionState(false, true);
	  outofspacedialog(curDisc->discName);
	}
      }
      else{
	curDisc->removeFolder(tmpFolder);
      }
    }
  }
}

FolderView::FolderView(GtkTreeView *tree_view_folder){
  tree_view = tree_view_folder;
  check_renderer = gtk_cell_renderer_toggle_new();
  discimage_renderer = gtk_cell_renderer_pixbuf_new();
  discassign_renderer = gtk_cell_renderer_text_new();
  folderimage_renderer = gtk_cell_renderer_pixbuf_new();
  folder_renderer = gtk_cell_renderer_text_new();

  check_column = gtk_tree_view_column_new_with_attributes ("", check_renderer, "active", 
							   CHECK_COLUMN1, NULL);;
  discimage_column = gtk_tree_view_column_new_with_attributes ("", discimage_renderer, "pixbuf", 
							       DISCIMAGE_COLUMN2, NULL);
  discassign_column = gtk_tree_view_column_new_with_attributes ("", discassign_renderer, "text", 
								DISCASSIGN_COLUMN3, NULL);
  folderimage_column = gtk_tree_view_column_new_with_attributes ("", folderimage_renderer, "pixbuf", 
								 FOLDERIMAGE_COLUMN4, NULL);
  folder_column = gtk_tree_view_column_new_with_attributes ("", folder_renderer, "text", 
							    FOLDER_COLUMN5, NULL);
  
  model = GTK_TREE_MODEL(gtk_list_store_new(5, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, 
					    G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_STRING));
}

FolderView::~FolderView(){  
  g_object_unref(model);
}


void FolderView::applyModel(){

  gtk_tree_view_set_model(tree_view, model);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), 0);

  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), check_column);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), discimage_column);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), discassign_column);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), folderimage_column);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), folder_column);

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
}

Folder *FolderView::findPath(string _path){
  Folder *tmpFolder = NULL;
    
  for(int i = 0; i < flatFolderList.size(); i++){
    if(flatFolderList[i]->getPath() == _path){
      tmpFolder = flatFolderList[i];
    }
  }
  return tmpFolder;
}

Folder *FolderView::findIter(GtkTreeIter _iter){
  Folder *tmpFolder = NULL;
  
  for(int i = 0; i < flatFolderList.size(); i++){
    if(flatFolderList[i]->iter.user_data == _iter.user_data){
      tmpFolder = flatFolderList[i];
    }
  }
  return tmpFolder;
}

void FolderView::createFlatList(Folder *curDir){

  for(int i = 0; i < curDir->children.size(); i++){
    flatFolderList.push_back(curDir->children[i]);
  }

  for(int i = 0; i < curDir->children.size(); i++){
    createFlatList(curDir->children[i]);
  }
}

Folder::Folder(char *_path){
  folderPath = _path;
  folderName = folderPath;
  assignedDisc = NULL;  
  bSelected = false;
  folderSize = 0;

  if(folderName[folderName.size() -1] == '/'){
    folderName = folderName.substr(0, folderName.size() - 1);
  }
  
  string tmpStr = "/";
  
  string::size_type pos = folderName.rfind(tmpStr,folderName.size() -1);
  if(pos != string::npos){
    folderName = folderName.substr(pos + 1, folderName.size()-1);
  }
  
  fParent = NULL;
  
  if(_path[strlen(_path)-1] == '/'){
    _path[strlen(_path)-1] = 0;
  }

  if(buildTree(_path)){
    folderPath = "none";
  }

}

Folder::Folder(){
  assignedDisc = NULL;  
  bSelected = false;
  folderSize = 0;
}

Folder::~Folder(){
  for(int i = 0; i < children.size(); i++){
    delete children[i];
  }
}


int Folder::toggleSelectionState(bool _state, bool _force){
  int rev = 0;
  if((fParent->assignedDisc == NULL) && ((!fParent->bSelected) || (_force))){
    bSelected = _state; 
    gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		       CHECK_COLUMN1, bSelected, -1); 
  }
  else{
    rev++;
  }
  
  if(bSelected){
    for(int i = 0; i < children.size(); i++){
      children[i]->toggleSelectionState(false, true);
    }
  }
  return rev;
}
void Folder::addChild(Folder* newChild){
 children.push_back(newChild);
 newChild->fParent = this;
}

Folder *Folder::getChild(int i){
  if(i > children.size()){
    return NULL;
  }
  else{
    return children[i];
  }
}

Folder *Folder::getParent(){
  return fParent;
}

double Folder::getForlderSize(){
  return folderSize;
}

double Folder::diskUsage(){
  double rev = 0;
  struct stat results;

  if(stat(folderPath.c_str(), &results) == 0){
    rev += results.st_size;
  }
  else{
    cout << "No such dir: " << folderPath << "\n";
  }

  DIR *theDir = NULL;
  theDir = opendir(folderPath.c_str());
 
  if(theDir == NULL){
    cout << "No such dir: " << folderPath << "\n";
  }
  else{
    dirent *theEnts;
    while((theEnts = readdir(theDir)) != NULL){
      string fname = theEnts->d_name;   
      if((fname != ".") && (fname != "..")){
	
	fname = folderPath + "/" + fname;
	
	if(stat(fname.c_str(), &results) == 0){
	  if(!S_ISDIR(results.st_mode)){
	    rev += results.st_size;
	  }
	}
	else{
	  cout << "No such file: " << fname << "\n";    
	}
      }
    }
  }
  
  for(int i = 0; i < children.size(); i++){
    rev += children[i]->diskUsage();
  }
  
  closedir(theDir);
  
  return rev;
}

void Folder::fillModel(GtkTreeModel *model){
  int depth = getDepth();
  vector<string> namePath;
  Folder *tmpFolder = this;
  int i = depth;
  while(i--){
    namePath.push_back(tmpFolder->folderName);
    tmpFolder = tmpFolder->fParent;
  }
  
  i = depth;
  string tmpSize = "(";
  double du = diskUsage();
  folderSize = du;
  du /= pow(double(2),double(20));
  stringstream convert;
  string tmpStr;
  convert << du;
  convert >> tmpStr;
  string tmpStr2 = ".";
  string::size_type pos = tmpStr.find(tmpStr2,0);
  tmpStr = tmpStr.substr(0,pos + 2);
  tmpSize += tmpStr;
  tmpSize += " mb) ";
  
  string tmpName;

  while(i--){
    tmpName +=  "/";
    tmpName += namePath[i];
  }  

  listPath = tmpName;
  tmpName = tmpSize + tmpName;

  if(fParent != NULL){

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       CHECK_COLUMN1, false, -1);
    
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
    		       DISCIMAGE_COLUMN2, project->cdromimage, -1);
    
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       DISCASSIGN_COLUMN3, "none", -1);
 
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       FOLDERIMAGE_COLUMN4, project->folderimage, -1);
    
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
		       FOLDER_COLUMN5, tmpName.c_str() , -1);
 
  }

  for(int i = 0; i < children.size(); i++){
    children[i]->fillModel(model);
  }
}

int Folder::getDepth(){
  int dLevel = 0;
  Folder *tmpFolder = fParent;

  while(tmpFolder != NULL){
    tmpFolder = tmpFolder->fParent;
    dLevel++;
  }
  return dLevel;
}

string Folder::getName(){
  return folderName;
}

void Folder::setName(string _name){
  folderName = _name;
}

string Folder::getPath(){
  return folderPath;
}

void Folder::setPath(string _path){
  folderPath = _path;
}

string Folder::getListPath(){
  return listPath;
}

void Folder::setListPath(string _path){
  listPath = _path;
}

string Folder::getBurnPath(){
  return burnPath;
}

void Folder::setBurnPath(string _path){
  burnPath = _path;
}


int Folder::buildTree(const char* dirName){
  DIR *theDir = NULL;
  theDir = opendir(dirName);
  int rev = 0;

  if(theDir == NULL){
    cout << "No such dir: " << dirName << "\n";
    return 1;
  }
  
  setPath(dirName);

  dirent *theEnts;
  while((theEnts = readdir(theDir)) != NULL){
    bool isDir = false;
    string nameDir = theEnts->d_name;
    if(theEnts->d_type == DT_DIR){
      isDir = true;
    }
    else{
      if(theEnts->d_type == DT_UNKNOWN){
	string filenamefull = dirName;
	filenamefull += "/";
	filenamefull += nameDir;
	struct stat buf;
	stat(filenamefull.c_str(), &buf);
	if(S_ISDIR(buf.st_mode)){
	  isDir = true;
	}
      }
    }

    if((isDir) && (nameDir != ".") && (nameDir != "..")){
      string newDirName = dirName;

      newDirName += "/";
      newDirName += theEnts->d_name;
      
      Folder *newFolder = new Folder;
      newFolder->setName(theEnts->d_name);
      addChild(newFolder);

      rev += newFolder->buildTree(newDirName.c_str());
    }
  }
  closedir(theDir);
  return rev;
}

void Folder::assignToNone(){
  assignedDisc = NULL;
  gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		     DISCASSIGN_COLUMN3, "none", -1);
  
  for(int i = 0; i < children.size(); i++){
    children[i]->assignToNone();
  }
}

void Folder::assignToDisc(Disc *aDisc){

  if(assignedDisc){
    assignedDisc->removeFolder(this);
  }
  
  assignedDisc = aDisc;

  if(aDisc){
    gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		       DISCASSIGN_COLUMN3, aDisc->getName(), -1);
  }
  else{
    gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		       DISCASSIGN_COLUMN3, "none", -1);
  }

  for(int i = 0; i < children.size(); i++){
    children[i]->assignToDisc(aDisc);
  }
}

void Folder::updateAssignment(Disc *aDisc){
  assignedDisc = aDisc;
  
  if(aDisc){
    gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		       DISCASSIGN_COLUMN3, aDisc->getName(), -1);
  }
  else{
    gtk_list_store_set(GTK_LIST_STORE(project->folderView->model), &iter,
		       DISCASSIGN_COLUMN3, "none", -1);
  }
  
  for(int i = 0; i < children.size(); i++){
    children[i]->assignToDisc(aDisc);
  }
}
